<?php

namespace FiloBlu\Flow\Block\Adminhtml\System\Config;

class MappingFields extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray
{

    /**
     * Enable the "Add after" button or not
     *
     * @var bool
     */
    protected $_addAfter = true;

    /**
     * @var \Magento\Store\Model\ResourceModel\Website\CollectionFactory
     */
    protected $_websitesCollection;
    /**
     * @var \FiloBlu\Flow\Model\ResourceModel\Channel\Collection
     */
    protected $_channelCollection;
    /**
     * @var null
     */
    protected $_websitesRenderer = null;


    /**
     * MappingFields constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Store\Model\ResourceModel\Website\CollectionFactory $collection
     * @param \FiloBlu\Flow\Model\ResourceModel\Channel\Collection $channell_collection
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Store\Model\ResourceModel\Website\CollectionFactory $collection,
        \FiloBlu\Flow\Model\ResourceModel\Channel\Collection $channell_collection,
        array $data = []
    )
    {
        $this->_websitesCollection = $collection;
        $this->_channelCollection = $channell_collection;
        parent::__construct($context, $data);
    }

    /**
     * @return array
     */
    protected function getChannelOptions() {

        $options = array();
        foreach ($this->_channelCollection as $channel) {
            $channel_data = json_decode($channel->getData('data'));
            if($channel_data->config->flow == "From\\Product") {
                $options[] = array(
                    "value" => $channel->getName(),
                    "label" => $channel->getName()
                );
            }
        }

        return $options;
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _prepareToRender()
    {

        $channel_options = $this->getChannelOptions();
        array_unshift($channel_options,  array('value' => '', 'label' => __('-- Select --')));

        $channelBlock=$this->getLayout()->createBlock('Magento\Framework\View\Element\Html\Select')
            ->setId($this->_getCellInputElementId('<%- _id %>','channel'))
            ->setName($this->_getCellInputElementName('channel'))
            ->setClass('select')
            ->setOptions($channel_options);

        $this->addColumn('channel', array(
            'label' => __('Masterfile'),
            'renderer' => $channelBlock
        ));


        $websitesBlock=$this->getWebsitesRenderer();
        $this->addColumn('website_id', array(
            'label' => __('Website'),
            'renderer' => $websitesBlock
        ));


        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
    }

    /**
     * @param \Magento\Framework\DataObject $row
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _prepareArrayRow(\Magento\Framework\DataObject $row)
    {
        $renderer=  $this->getWebsitesRenderer();

        $column_values=$row->getColumnValues();
        unset($column_values[$row->getData("_id")."_website_id"]);
        $row->setData("column_values", $column_values);

        //Set selected values for multiselect using option_extra_attrs
        $websites = $row->getWebsite_id();
        if($websites){
            $options = [];
            foreach ($websites as $website) {
                $options['option_' . $renderer->calcOptionHash($website)]
                    = 'selected="selected"';
            }
            $row->setData('option_extra_attrs', $options);
        }
    }

    /**
     * @return |null
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function getWebsitesRenderer()
    {
        if (!$this->_websitesRenderer) {

            $websites_options = $this->_websitesCollection->create()->toOptionArray();
            array_unshift($websites_options,  array('value' => '', 'label' => __('-- Select --')));

            $this->_websitesRenderer = $this->getLayout()->createBlock(
                'Magento\Framework\View\Element\Html\Select',
                '',
                ['data' => ['is_render_to_js_template' => true]]
            )
                ->setId($this->_getCellInputElementId('<%- _id %>','website_id'))
                ->setName($this->_getCellInputElementName('website_id')."[]")
                ->setClass('multiselect')
                ->setOptions($websites_options)
                ->setExtraParams('style="width:160px" multiple="multiple" size="5" ');

        }
        return $this->_websitesRenderer;
    }

}
