<?php

namespace FiloBlu\Flow\Model\Actions\From;

/**
 * Class AbstractWebsite
 * @package FiloBlu\Flow\Model\Actions\From
 */
abstract class AbstractWebsite
{

    /**
     * @var string
     */
    CONST ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * @var \Magento\Catalog\Model\Product
     */
    protected $productFactory;

    /**
     * @var array|\Magento\Store\Model\StoreRepository
     */
    protected $websites = [];

    /**
     * @var \Magento\Store\Model\WebsiteRepository
     */
    protected $websiteRepository;

    /**
     * AbstractWebsite constructor.
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Store\Model\WebsiteRepository $websiteRepository
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Store\Model\WebsiteRepository $websiteRepository
    )
    {
        $this->productFactory = $productFactory;
        $this->websiteRepository = $websiteRepository;
        $this->websites = $this->getWebsites();
    }

    /**
     * @return array
     */
    public function getWebsites()
    {
        $websites = $this->websiteRepository->getList();
        $websiteIdArray = [];
        $websiteCodeArray = [];
        foreach ($websites as $website) {
            $websiteIdArray[$website["website_id"]] = $website["website_id"];
            $websiteCodeArray[$website["code"]] = $website["website_id"];
        }

        return ['id' => $websiteIdArray, 'code' => $websiteCodeArray];
    }


    /**
     * @param $prod
     * @param $value
     * @param $storeId
     * @return bool
     */
    public function process($prod, $value, $storeId)
    {

        $websites = explode(',',$value);

        if(count($websites) === 0){
            return false;
        }

        $websiteToProcess = $this->getNormalizedWebsiteArray($websites);

        if(count($websiteToProcess)){
            $product_id = $this->productFactory->create()->getIdBySku($prod->getSku());
            $product = $this->productFactory->create()->setStoreId(0)->load($product_id);
            $finalWebsiteArray = $this->getFinalWebsiteArray($websiteToProcess,$product->getWebsiteIds());
            $product->setWebsiteIds($finalWebsiteArray);
            $product->save();
        }

    }

    /**
     * @param $websites
     * @return array
     */
    protected function getNormalizedWebsiteArray($websites){

        $websiteToProcess = [];
        foreach($websites as $website){
            if(is_numeric($website)){
                if(key_exists($website, $this->websites['id'])){
                    $websiteToProcess[] = $this->websites['id'][$website];
                }
            }
            else {
                if(key_exists($website, $this->websites['code'])){
                    $websiteToProcess[] = $this->websites['code'][$website];
                }
            }
        }

        return array_unique($websiteToProcess);
    }

    /**
     * @param $websites
     * @return mixed
     */
    abstract protected function getFinalWebsiteArray($websites,$existingWebsites);

}
