<?php

namespace FiloBlu\Flow\Model\From;

use Exception;
use FiloBlu\Flow\Helper\Data;
use Magento\Catalog\Model\ProductFactory;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Model\Context;
use Magento\Framework\Registry;
use Magento\Store\Model\StoreManagerInterface;
use Monolog\Logger;

/**
 * Class Price
 * @package FiloBlu\Flow\Model\From
 */
class Price extends AbstractFrom
{
    /**
     * @var Logger
     */
    protected $_logger;
    /**
     * @var ProductFactory
     */
    protected $_productFactory;
    /**
     * @var mixed
     */
    protected $_productAction;
    /**
     * @var mixed
     */
    protected $_configurableAction;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var Data
     */
    protected $_helper_flow;

    /**
     * Price constructor.
     * @param Context $context
     * @param Registry $registry
     * @param Data $helper_flow
     * @param StoreManagerInterface $storeManager
     * @param ProductFactory $productFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        Data $helper_flow,
        StoreManagerInterface $storeManager,
        ProductFactory $productFactory,
        array $data = []
    )
    {
        $this->_helper_flow = $helper_flow;
        $this->_storeManager = $storeManager;
        $this->_productFactory = $productFactory;
        $this->_productAction = ObjectManager::getInstance()->get(\Magento\Catalog\Model\Product\Action::class);
        $this->_configurableAction = ObjectManager::getInstance()->create(\Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable::class);
        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init(\FiloBlu\Flow\Model\ResourceModel\From\Price::class);
    }

    /**
     * @return $this|bool|mixed
     * @throws Exception
     */
    public function process()
    {

        $productSku = $this->getSku();
        $paese = $this->getPaese();
        $prezzo = $this->getPrezzo();

        if (ctype_space($prezzo)) {
            $message = sprintf("Empty price! Skipping! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }

        $allStore = $this->_helper_flow->getStoreInWebsiteId();

        if ($paese === null) {
            $paese = 'admin';
        }
        $storeId = $allStore[$paese]['store_id'];

        $productId = $this->_productFactory->create()->getIdBySku($productSku);

        if (!$productId) {
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }

        if ($productId) {

            $pids[] = $productId;

            // salvo il prezzo sul prodotto configurabile padre
            $parent_product_id = $this->_configurableAction->getParentIdsByChild($productId);

            if (!empty($parent_product_id)) {
                $pids = array_merge($pids, $parent_product_id);
            }

            try {
                $this->_productAction->updateAttributes($pids, ['price' => $prezzo], $storeId);
            } catch (Exception $e) {
                throw new Exception(sprintf("%s .Problem while processing price. Store Id '%s, Product Ids '%s',  Attribute data '%s'",
                    $e->getMessage(),
                    $storeId,
                    implode(',', $pids),
                    print_r($prezzo, true))
                );
            }
        }

        return $this;
    }

    /**
     * @param $file
     * @return bool|mixed
     */
    public function sendErrorNotifications($file)
    {
        return true;
    }
}
